function [forecasts,forecerrors] = predict(model,xstart,n);
% compute forecasts and forecast errors (homoskedastic case)
%
% syntax is 
%
% [forecasts,ferrorvariances] = predict(model,xstart,n);
%
% INPUTS: 
%  model is an ssmodel object;
%  n is the number of leads;
%  xstart is a (#states x #initialstates) array of initial states
%
% OUTPUTS:
%  forecasts is a  #observables x #initialstates x #leads array 
%    that contains the forecasts from the model; 
%  ferrorvariances is a  #observables x #observables x #leads array 
%   that contains the forecast error variances (that do not depend on 
%   initial states!)


A = model.phix;
C = model.phiy;
V = model.varw;
sigx = model.sigx;
sigy = model.sigy;

starts = size(xstart,2);
xs = size(A,1);
ys = size(C,1);
forecasts = zeros(ys,starts,n);
forecerrors = zeros(ys,ys,n);

% construct forecasts

for j=1:n;    % loop is over shocks that realize in t+j
    forecasts(:,:,j) = C * (A^(j-1))*xstart;    
    forecerrors(:,:,j) = squeeze(forecerrors(:,:,j)) + sigy*V*sigy'; % obs noise in t+j contributes to j period ahead error 
    for k=j+1:n; % state noise in t+j contrib to all errors in t+k > t+j
        forecerrors(:,:,k) = squeeze(forecerrors(:,:,k)) + C*(A^(k-1-j))*sigx*V*sigx'*(A^(k-1-j))'*C';
    end;    
end;


% logic: j=1 gets only obs noise
%        j=2 gets state noise, but without A
%        j=3 
% matrix 2 is affected directly as obs noise is added, but was also affected when 
%   j=1, where state noise was added with A to the power 0!